@@ -179,7 +179,7 @@ beamline_scope = freephil.parse(
       wavelength = 0.979590
         .type = float
         .help = "Wavelength of incident beam, angstroms"
-      flux = 268717230611.358
+      flux = None
         .type = float
         .help = "Flux of incident beam, ph / s"
     }
@@ -203,6 +203,17 @@ timestamp_scope = freephil.parse(
     """
 )
 
+pump_probe_scope = freephil.parse(
+    """
+    pump_exp = None
+      .type = float
+      .help = "Pump exposure time."
+    pump_delay = None
+      .type = float
+      .help = "Pump delay"
+    """
+)
+
 if __name__ == "__main__":
     print(detector_scope.as_str())
     print(module_scope.as_str())
